/* --------------------------------------------------------
   | FILE    : mark_c10.c                                 |
   | PROJECT : Low Cost PC Based Oscilloscope             |
   | DESC    : Test RAM Chip                              |
   | ==================================================== |
   | DATE    : 21/03/2002                                 |
   | BY      : Colin K McCord                             |
   | VERSION : 1.2                                       |
   -------------------------------------------------------- */


#include <16F877.h>
#device PIC16F877 *=16 ADC=10

// use #device adc = 10 to implement a 10-bit conversion, 
// otherwise the default is 8-bits.


#fuses HS,NOWDT,NOPROTECT,NOLVP,NOBROWNOUT

#byte   PORTB = 0x06	// PortB lives in File 6 (data bus)
#byte   PORTC = 0x07	// PortC lives in File 7 (address bus A12 to A8, RX, TX & R/W)
#byte 	PORTD = 0x08    // PortD lives in File 8 (address bus A0 to A7)
#byte 	PORTE = 0x09	// PortE lives in File 9 (used for DIP switches)

#bit 	RW = PORTC.5	// RW = 0 for Write, and 1 for read


/* Note 20MHz clock must be used for 115,000 bps, the % error is to large at slower speeds */
#use delay(clock = 20000000)
#use rs232(baud=9600, xmit = PIN_C6, rcv = PIN_C7, parity = N, bits = 8)

#use 	fast_io(B)	// Fast access to PortB (don't fiddle with TRISB)
#use 	fast_io(C)	// Fast access to PortB (don't fiddle with TRISB)
#use 	fast_io(D)	// Fast access to PortD (don't fiddle with TRISD)
#use	fast_io(E)	// Fast access to PortE (don't fiddle with TRISE)


/* Forward declaration of functions */
void SetBaudRate();
void WriteRAM(long int address, char data);
char ReadRAM(long int address);

main()
{	
	long int i;
	long int pass, fail;
	set_tris_c(0xC0);	// TRISD = 11000000; RC0 to RC5 TTL Outputs (bits 7&6 must be set for UART to work)	
	set_tris_d(0x00);	// TRISD = 00000000; RD0 to RD7 TTL Outputs
	port_b_pullups(FALSE); 	// Don't use internal pull up resistors

	SetBaudRate();


	while(TRUE)
	{
		delay_ms(5000); // 5 Sec delay

		printf("TEST RAM CHIP...");


		/** RAM Test 1 write 0xFF into all locations and check **/
		printf("\n\nTest 1 - Fill RAM with 0xFF --->");
		pass = 0;	// Reset Pass counter
		fail = 0;	// Reset Fail counter

		for (i=0;i<8192;i++)	// Write 0xFF to all locations
		{	
			WriteRAM(i,0xFF);
		}

		for (i=0;i<8192;i++)
		{
			if(ReadRAM(i) == 0xFF) pass++;
			else fail++;
		}
		
		printf(" Passes = %lu, Fails = %lu", pass,fail);


		/** RAM Test 2 write 0x00 into all locations and check **/
		printf("\n\nTest 2 - Fill RAM with 0x00 --->");
		pass = 0;	// Reset Pass counter
		fail = 0;	// Reset Fail counter

		for (i=0;i<8192;i++)	// Write 0x00 to all locations
		{	
			WriteRAM(i,0x00);
		}

		for (i=0;i<8192;i++)
		{
			if(ReadRAM(i) == 0x00) pass++;
			else fail++;
		}
		
		printf(" Passes = %lu, Fails = %lu", pass,fail);


		/** RAM Test 3 write LSB of address into all locations and check **/
		printf("\n\nTest 3 - Fill RAM with LSB of Address --->");
		pass = 0;	// Reset Pass counter
		fail = 0;	// Reset Fail counter

		for (i=0;i<8192;i++)	// Write LSB of address to all locations
		{	
			WriteRAM(i,(char)i);
		}

		for (i=0;i<8192;i++)
		{
			if(ReadRAM(i) == (char)i) pass++;
			else fail++;
		}
		
		printf(" Passes = %lu, Fails = %lu", pass,fail);


		printf("\n\nEnd of RAMTEST, Hopefully work can now start on storage mode.\n\n");

	}
	sleep();
}


void SetBaudRate()
{
	// Verison 1.1, 22/3/2002, by CKM

	set_tris_e(0x17);	// TRISE = 00010111; RE2,RE1 and RE0 Inputs
				// Parrell slave mode is ON, e.g. TTL Inputs, on Port E and D

	switch(PORTE & 0x07)	// Read dip switches and setup baud rate
	{
		case 0: set_uart_speed(4800);   break;
		case 1: set_uart_speed(9600);   break;
		case 2: set_uart_speed(14400);  break;
		case 3:	set_uart_speed(19200);	break;
		case 4: set_uart_speed(32768);  break;
		case 5: set_uart_speed(38400);  break;
		case 6: set_uart_speed(57600);  break;
		case 7: set_uart_speed(115200); break;
	}

	set_tris_e(0x07);  // Turn OFF Parral Slave Mode as this affects port D (Address Bus)
}

void WriteRAM(long int address, char data)
{
	// 20/03/2002, version 1.0 by Colin McCord

	set_tris_b(0x00);	// TRISE = 00000000; RB0 to RB7 TTL Outputs (data bus)

	RW = 1;			// Don't Write until address and data bus is setup.

	// Set Address
	PORTD = (char)address;			// Set A7..A0
	PORTC = ((PORTC & 0xE0) | (address>>8)); 	// Set A12..A8

	PORTB = data;		// Put data on the data bus

	
	delay_cycles(1);	// 200nS delay, same as NOP, (1/20,000,000)*4*1) = 200nS
				// make sure address & data is valid before write

	RW = 0; 		// Write data to specified address

	delay_cycles(1);	// 200nS delay (same as NOP), make sure write is finished
	RW = 1;			// Back to Read Mode
}

char ReadRAM(long int address)
{
	// 20/03/2002, version 1.0 by Colin McCord

	set_tris_b(0xFF);	// TRISE = 11111111; RB0 to RB7 TTL Inputs

	RW = 1;			// Put RAM chip in Read Mode

	// Set Address
	PORTD = ((char)address);			// Set A7..A0
	PORTC = ((PORTC & 0xE0) | (address>>8)); 	// Set A12..A8

	delay_cycles(1);	// 200nS delay, same as NOP, (1/20,000,000)*4*1) = 200nS
				// make sure data is valid before reading

	return PORTB;
}
