; --------------------------------------------------------
; | FILE    : rs232.asm                                  |
; | DESC    : RS232 subroutines                          |
; | PROJECT : Low Cost PC Based Oscilloscope             |
; | ==================================================== |
; | DATE    : 12/02/2002                                 |
; | BY      : Colin K McCord                             |
; | VERSION : 1.1                                        |
; --------------------------------------------------------

	#define BAUD .4800	; Put baud rate here (e.g. 4800bps = 4800)
	#define XTAL .4		; Put Crystal fequency here (e.g. 8MHz = 8)	
	#define	X  ((XTAL * .1000000)/(.64 * BAUD))-.1
	
	; ***********************************************************
	; .# for base 10 e.g. 10dec = .10
	; ***********************************************************


DATA_TX		equ 	20h	; TX Register
DATA_RX		equ 	21h	; RX Register
ERR			equ		22h	; RX Error Register


; --------------------------------------------------------
; | FUNCTION: Initialises USART for RS232 Comm..         |
; | RESOURCE: PIC USART                                  |
; | ENTRY   : None                                       |
; | EXIT    : USART Initialised. ready for RS232 comm... |
; | ==================================================== |
; | DATE    : 12/02/2002                                 |
; | BY      : Colin K McCord                             |
; | VERSION : 1.0                                        |
; --------------------------------------------------------


INITCOMS	

	bsf		STATUS,RP0	; Change to Bank1.
	bsf		TRISC,7		; RX pin is set as an input.
	bcf		TRISC,6		; TX is set as an output.
	movlw	X			; Move X to Baud Rate Generator.
	movwf	SPBRG	
	movlw	b'00100000'	; 8 data bits, TX enabled.
	movwf	TXSTA		; Low speed SPBRG mode.
	bcf		STATUS,RP0	; Back again to Bank0.
	movlw	b'10010000'	; USART enabled, 8 data bits.
	movwf	RCSTA		; Receiver enabled.
	return	


; --------------------------------------------------------
; | FUNCTION: Transmits one char over RS232 using USART  |
; | RESOURCE: PIC USART                                  |
; | ENTRY   : 8-bit datum in DATA_TX                     |
; | EXIT    : Contents of DATA_TX unchanged, byte TXed   |
; | ==================================================== |
; | DATE    : 12/02/2002                                 |
; | BY      : Colin K McCord                             |
; | VERSION : 1.0                                        |
; --------------------------------------------------------

PUTCHAR

	btfss	PIR1,TXIF	; Check, is TX buffer full?
	goto	PUTCHAR		; IF not THEN try again.
	movf	DATA_TX,w	; ELSE get datum.
	movwf	TXREG		; and copy to USART TX register.
	return


; --------------------------------------------------------
; | FUNCTION: Receives a char form RS232 using USART     |
; | RESOURCE: PIC USART                                  |
; | ENTRY   : None                                       |
; | EXIT    : DATA_RX holds the received byte            |
; |         : ERR is 00 if no error, Framing ERR = -1,   |
; |         : ERR = -2 if overflow and ERR = -3 if both. |
; | ==================================================== |
; | DATE    : 12/02/2002                                 |
; | BY      : Colin K McCord                             |
; | VERSION : 1.0                                        |
; --------------------------------------------------------

GETCHAR

	clrf	ERR			; Zero flag byte.
	btfss	PIR1, RCIF	; Check, is there a char ready?
	goto	GETCHAR		; IF not THEN try again.

; Error return
	btfss 	RCSTA, FERR	; Was there a Framing error ?
	goto	CHECK_OERR	; IF not THEN check for OIverflow.
	movlw	-1			; ELSE record a Framing error.

CHECK_OERR

	btfsc	RCSTA,OERR	; Check for Overflow error.
	goto	GET_EXIT	; IF none THEN complete.
	decf	ERR,f		; Otherwise register error
	decf	ERR,f		
	bcf		RCSTA,CREN	; and rest the logic
	bsf		RCSTA,CREN

GET_EXIT

	movf	RCREG,w		; Get datum.
	movwf	DATA_RX		; and put away.
	return

	
