; --------------------------------------------------------
; | FILE    : mark_4.asm                                 |
; | PROJECT : Low Cost PC Based Oscilloscope             |
; | DESC    : Read CH1 ADC, transmit result through      |
; |         : RS232 using the real-time frame format.    |          
; | ==================================================== |
; | DATE    : 13/02/2002                                 |
; | BY      : Colin K McCord                             |
; | VERSION : 1.1                                        |
; --------------------------------------------------------			

	#include <p16f877.inc>	; Using PIC16F877

	org 0x0000	; Reset Vector
	goto MAIN	

	org 0x0020	; Start of program

	
	#include "rs232.asm" 	; Include RS232 Subroutines.
	#include "adc.asm"	; Include ADC Subroutines.


REAL_TIME_B1	equ	26h	; Real-Time byte 1.
REAL_TIME_B2	equ	27h	; Real-Time byte 2.

MAIN
	call 	INITCOMS	; Initialise RS232 Communications.
	call 	INIT_ADC	; Initialise ADC (ch1).

LOOP
	call 	READ_ADC	; Sample channel 1.	
	call 	ADC_TO_REALTIME	; Convert ADC double-byte to real-time frame structure

	movf 	REAL_TIME_B1,w	; Move byte 1 to the working register.
	movwf	DATA_TX		; Perpare to transmit byte 1.
	call 	PUTCHAR		; Transmit byte 1

	movf	REAL_TIME_B2,w	; Move byte 2 to the working register.
	movwf	DATA_TX		; Perpare to transmit byte 1.
	call	PUTCHAR		; Transmit byte 2.


	goto 	LOOP		; Loop Forever..............



; --------------------------------------------------------
; | FUNCTION: Convert ADC value to Real-Time Frame       |
; | RESOURCE: None                                       |
; | ENTRY   : ADC_HI, ADC_LO                             |
; | EXIT    : REAL_TIME_B1, REAL_TIME_B2                 |
; | ==================================================== |
; | DATE    : 13/02/2002                                 |
; | BY      : Colin K McCord                             |
; | VERSION : 1.0                                        |
; --------------------------------------------------------

ADC_TO_REALTIME
	
	; ADC Structure (right justified): -

	;	        ADC_HI		              ADC_LO
	;       -------------------------   -------------------------
	;	|0 |0 |0 |0 |0 |0 |d9|d8|   |d7|d6|d5|d4|d3|d2|d1|d0|
	;	-------------------------   -------------------------
	;        7  6  5  4  3  2  1  0      7  6  5  4  3  2  1  0
		
	;				|   |
	;				|   |
	;				\   /
	;				 \ /

	; Real-Time Frame Structure: -

	; 	       REAL_TIME_B1                 REAL_TIME_B2
	; 	-------------------------   -------------------------
	; 	|A |B |C |d9|d8|d7|d6|d5|   |A |B |C |d4|d3|d2|d1|d0|
	; 	-------------------------   -------------------------
        ;        7  6  5  4  3  2  1  0      7  6  5  4  3  2  1  0

	;	A = 0, for byte 1 (d9 to d5), A = 1 for byte 2 (d4 to d0)
	; 	BC = 00 for CH1, BC = 01 for CH2, BC = 10 for CH3, BC = 11 for CH4

	clrf	REAL_TIME_B1	; Clear File
	clrf	REAL_TIME_B2	; Clear File

	btfsc	ADC_HI,1	; Check d9, Skip if Clear.
	bsf	REAL_TIME_B1,4	; Else Set d9 in REAL_TIME_B1.
	btfsc	ADC_HI,0	; Check d8, Skip if Clear.
	bsf	REAL_TIME_B1,3	; Else Set d8 in REAL_TIME_B1.
	btfsc	ADC_LO,7	; Check d7, Skip if Clear.
	bsf	REAL_TIME_B1,2	; Else Set d7 in REAL_TIME_B1.
	btfsc	ADC_LO,6	; Check d6, Skip if Clear.
	bsf	REAL_TIME_B1,1	; Else Set d6 in REAL_TIME_B1.
	btfsc	ADC_LO,5	; Check d5, Skip if Clear.
	bsf	REAL_TIME_B1,0	; Else Set d5 in REAL_TIME_B1.

	btfsc	ADC_LO,4	; Check d4, Skip if Clear.
	bsf	REAL_TIME_B2,4	; Else Set d4 in REAL_TIME_B2.
	btfsc	ADC_LO,3	; Check d3, Skip if Clear.
	bsf	REAL_TIME_B2,3	; Else Set d3 in REAL_TIME_B2.
	btfsc	ADC_LO,2	; Check d2, Skip if Clear.
	bsf	REAL_TIME_B2,2	; Else Set d2 in REAL_TIME_B2.
	btfsc	ADC_LO,1	; Check d1, Skip if Clear.
	bsf	REAL_TIME_B2,1	; Else Set d1 in REAL_TIME_B2.
	btfsc	ADC_LO,0	; Check d0, Skip if Clear.
	bsf	REAL_TIME_B2,0	; Else Set d0 in REAL_TIME_B2.

	bsf	REAL_TIME_B2,7	; A=1, e.g. byte 2

	; note B&C for B1 and B2 = 0 hence reading for channel 1
	return

	END

