/* --------------------------------------------------------
   | FILE    : mark_c8.c                                  |
   | PROJECT : Low Cost PC Based Oscilloscope             |
   | DESC    : Read CH1 & CH2, transmit result through    |
   |         : RS232 using the real-time frame format.    |
   |         : Baud rate set by dip-switches.             |
   | ==================================================== |
   | DATE    : 05/03/2002                                 |
   | BY      : Colin K McCord                             |
   | VERSION : 1.4                                        |
   -------------------------------------------------------- */


#include <16F877.h>
#device PIC16F877 *=16 ADC=10

// use #device adc = 10 to implement a 10-bit conversion, 
// otherwise the default is 8-bits.


#fuses HS,NOWDT,NOPROTECT,NOLVP

#byte 	PORTE = 0x09	// PortE lives in File 9

/* Note 20MHz clock must be used for 115,000 bps, the % error is to large at slower speeds */
#use delay(clock = 20000000)
#use rs232(baud=9600, xmit = PIN_C6, rcv = PIN_C7, parity = N, bits = 8)

#use	fast_io(E)		// Fast access to PortE (don't fiddle with TRISE)

/* Forward declaration of functions */
void SetBaudRate();

main()
{
	// NOTE: by default in CCS all var's are unsigned
	long int adcValue; 		// 16-bit storage for ADC reading
	char adcHI,adcLO;		// 8-bit storage for real-time frames.

	set_tris_e(0x17);		// TRISE = 00010111; RE2,RE1 and RE0 TTL Inputs
	SetBaudRate();

	setup_adc_ports(A_ANALOG);	// RA0 - RA4 Analog, RE0 - RE2 digital
	setup_adc(ADC_CLOCK_DIV_32);	
	set_adc_channel(0);
	
	delay_us(20);	// Delay for sampling cap to charge 

	while(TRUE)
	{
		adcValue = read_adc();	// Get ADC reading

		/* Convert 16-bit adcValue to Real-time frame structure, CH1 */
		adcHI = (char)((adcValue >> 5)& 0x1f);	// 0|0|0|d9|d8|d7|d6|d5
		adcLO = (char)((adcValue & 0x1f)|0x80);	// 1|0|0|d4|d3|d2|d1|d0
			
		putc(adcHI);	// Transmit Byte 1 (d9...d5)
		putc(adcLO);	// Transmit Byte 2 (d4...d0)
	
		set_adc_channel(1);
		delay_us(20);	// Delay for sampling cap to charge 

		adcValue = read_adc();	// Get ADC reading

		/* Convert 16-bit adcValue to Real-time frame structure, CH2 */
		adcHI = (char)(((adcValue >> 5)& 0x1f)|0x20);	// 0|0|1|d9|d8|d7|d6|d5
		adcLO = (char)((adcValue & 0x1f)|0xA0);		// 1|0|1|d4|d3|d2|d1|d0
			
		putc(adcHI);	// Transmit Byte 1 (d9...d5)
		putc(adcLO);	// Transmit Byte 2 (d4...d0)


		set_adc_channel(0);
		delay_ms(10);	// Preset delay, repeat every 10ms, thats 100 Hz
	}
}

void SetBaudRate()
{
	switch(PORTE & 0x07)	// Read dip switches and setup baud rate
	{
		case 0: set_uart_speed(4800);   break;
		case 1: set_uart_speed(9600);   break;
		case 2: set_uart_speed(14400);  break;
		case 3:	set_uart_speed(19200);	break;
		case 4: set_uart_speed(32768);  break;
		case 5: set_uart_speed(38400);  break;
		case 6: set_uart_speed(57600);  break;
		case 7: set_uart_speed(115200); break;
	}
}
