/* --------------------------------------------------------
   | FILE    : mark_c4.c                                  |
   | PROJECT : Low Cost PC Based Oscilloscope             |
   | DESC    :    |          
   | ==================================================== |
   | DATE    : 20/02/2002                                 |
   | BY      : Colin K McCord                             |
   | VERSION : 1.1                                        |
   -------------------------------------------------------- */


#include <16F877.h>
#device PIC16F877 *=16 ADC=10

// use #device adc = 10 to implement a 10-bit conversion, 
// otherwise the default is 8-bits.


#fuses HS,NOWDT,NOPROTECT,NOLVP
#use delay(clock = 4000000)
#use rs232(baud=9600, xmit = PIN_C6, rcv = PIN_C7, parity = N, bits = 8)


enum state
{
	init,
	realtime,
	storage,
	charge_delay,
	sampleADC
} mstate;


char Setup_Reg1 = 0x01;	// default value is 00000001
                       	// Real-time mode, RATE Multiplier = X1, Sample when PIC is awake,
			// channel 1 enabled, channel 2-4 disabled.

char Setup_Reg2 = 0x00;	// default value is 00000000


long int CRC16(char *message, int length);
void Control_Protocol();

main()
{
	// NOTE: by default in CCS all var's are unsigned.
	long int adcValue; 	// 16-bit storage for ADC reading.
	char adcHI,adcLO;	// 8-bit storage for real-time frames.
	char ch;		// Selected ADC channel (0 to 3).	
	char oldch;
	long int sample_rate;	// Sample rate in uS	

	setup_adc_ports(A_ANALOG);	// RA0 - RA4 Analog, RE0 - RE2 digital
	setup_adc(ADC_CLOCK_INTERNAL);

	mstate = init;
		

	/* Temp set sample rate to 100Hz */
	sample_rate = 10000; // (1/100 = 10ms = 10,000us)

	while(TRUE)
	{
		/* Check if there is a character waiting to be read in the UART */ 
		if(kbhit() == TRUE) Control_Protocol();

		switch(mstate)
		{

			case init:
			
				if (bit_test(Setup_Reg1,7) == 0) mstate = realtime;
				else                             mstate = storage;
				
				/* Increments every 1uS when running at 4MHz */
				/* Note timer input clock = FOSC/4           */
				setup_timer_2 (T2_DIV_BY_1,255,16);		// 8-bit timer
				setup_timer_1 (T1_DIV_BY_1 | T1_INTERNAL);	// 16-bit timer

				set_timer1(0);		// Reset sampling timer.

				ch = 0;		// Channel 1 selected
				oldch = 255;
				break;
			

			case realtime:
				/* Check that selected channel is enabled */
				if(!bit_test(Setup_Reg1,ch)) 
				{
					ch++;
					if(ch > 3) ch = 1;
					break;
				}
				
				mstate = sampleADC; // next state

				/* Only set adc channel if, it has not been setup yet */
				if(oldch != ch)
				{
					set_adc_channel(ch);	// Setup ADC channel
					set_timer2(0);		// reset timer 0;

					/* Note need 20uS delay for sampling cap to charge */
					mstate = charge_delay;
				}
				
				break;


			case charge_delay:
				/* Wait 20uS for sampling cap to charge */
				if(get_timer2() > 20) mstate = sampleADC;
				break; 
			



			case sampleADC:
				if (get_timer2() > sample_rate) 
				{
					adcValue = read_adc();	// Get ADC reading
					set_timer1(0);		// Reset sampling timer.

					/* Convert 16-bit adcValue to Real-time frame structure */
					adcHI = (char)((adcValue >> 5)& 0x1f);	// 0|0|0|d9|d8|d7|d6|d5
					adcLO = (char)((adcValue & 0x1f)|0x80);	// 1|0|0|d4|d3|d2|d1|d0
					
					putc(adcHI);	// Transmit Byte 1 (d9...d5)
					putc(adcLO);	// Transmit Byte 2 (d4...d0)
					
					oldch = ch;

					mstate = realtime;
				}
				break;
			
				
			default:
				mstate = init;
				break;

		}
	}



}

long int CRC16(char *message,int length)
{
	/* 16-bit var's */
	long int CRC = 0xFFFF,	     		/* CRC initialised with all '1's  */
		       i,            		/* Loop Control                   */
		       temp;	        	/* Temp for swap                  */

	while (length--)                	/* Loop until all bytes calulated */
	{
		CRC=CRC^(*message++);

		for (i=0;i++<8;)		/* Shift all 8-bits               */
		{
			if (CRC & 0x0001) CRC = (CRC>>1) ^0xA001;
			else CRC>>=1;
		}
	}

	temp = CRC & 0xFF;                 	/* Temp storage for swap          */
	CRC = ((CRC>>8)&0xFF) + (temp<<8); 	/* Swap high amd low byte         */

	return (CRC);  		     	     	/* Return calulated 16-bit CRC    */
}

void Control_Protocol()
{

}
